<?php

namespace Lumpik;

use FML\ManiaCode;
use FML\ManiaLink;
use FML\Controls\Frame;
use FML\Controls\Quad;
use FML\Controls\Quads\Quad_Icons64x64_1;
use FML\Controls\Labels\Label_Text;
use FML\Controls\Labels\Label_Button;
use FML\Controls\Control;
use ManiaControl\Logger;
use ManiaControl\ManiaControl;
use ManiaControl\Callbacks\Callbacks;
use ManiaControl\Callbacks\CallbackListener;
use ManiaControl\Callbacks\CallbackManager;
use ManiaControl\Callbacks\TimerListener;
use ManiaControl\Callbacks\TimerManager;
use ManiaControl\Callbacks\Structures\ArmorEmptyStructure;
use ManiaControl\Callbacks\Structures\NearMissStructure;
use ManiaControl\Callbacks\Structures\CaptureStructure;
use ManiaControl\Callbacks\Structures\PlayerHitStructure;
use ManiaControl\Plugins\Plugin;
use ManiaControl\Maps\MapManager;
use ManiaControl\Players\Players;
use ManiaControl\Players\Player;
use ManiaControl\Players\PlayerManager;
use ManiaControl\Manialinks\ManialinkPageAnswerListener;
use ManiaControl\Manialinks\IconManager;
use ManiaControl\Manialinks\ManialinkManager;
use ManiaControl\Commands\CommandListener;


// Include FML
require_once __DIR__ . '/../../libs/FML/autoload.php';

// DEBUG MODE
define('ManiaControl\DEV_MODE', false);



/**
 * Sound Events Manager (SEM) plugin.
 *
 * For any bug/suggestion : https://forum.maniaplanet.com/viewtopic.php?f=479&t=30285
 *
 * @author Lumpik
 */



class SoundEventsManager implements CallbackListener, CommandListener, TimerListener, ManialinkPageAnswerListener, Plugin {
	
    /**
     * Constants
     */
    const ID			= 70;
    const VERSION		= 1.0;
    const NAME			= 'Sound Events Manager';
    const AUTHOR		= 'Lumpik';
	const DESCRIPTION	= 'Play sounds on triggers, such as kills, beginning of round, etc...';
	
	// ManiaLinks ID
	const ML_ID			= 'SEM.ManiaLink.Id';
	const ML_MESSAGE_ID	= 'SEM.ManiaLink.Message.Id';
	const ML_RANKING_ID	= 'SEM.ManiaLink.Ranking.Id';
	
	
	// Icons for enabling/disabling sounds
	// Copied from http://www.softicons.com/search?search=iconza+sound&x=0&y=0
	const ENABLE_DISABLE_SEM			= 'SEM.Enable.Disable.ID';
	const SEM_ICON_SOUND_ON				= 'sound_on_black.png';
	const SEM_ICON_SOUND_ON_PATH		= 'http://mm2monde.fr/Plugins/SoundEventsManager/Icones';
	const SEM_ICON_SOUND_ON_MOVER		= 'sound_on_grey.png';
	const SEM_ICON_SOUND_ON_MOVER_PATH	= 'http://mm2monde.fr/Plugins/SoundEventsManager/Icones';
	const SEM_ICON_SOUND_OFF			= 'sound_off_black.png';
	const SEM_ICON_SOUND_OFF_PATH		= 'http://mm2monde.fr/Plugins/SoundEventsManager/Icones';
	const SEM_ICON_SOUND_OFF_MOVER		= 'sound_off_grey.png';
	const SEM_ICON_SOUND_OFF_MOVER_PATH	= 'http://mm2monde.fr/Plugins/SoundEventsManager/Icones';
	
	// DEBUG + VOLUME parameters
	const DEBUG		= '00a. DEBUG ';
	const VOLUME	= '00b. VOLUME';
	
	// MESSAGE
	const SEM_MESSAGES_POS_X	= '00c. MESSAGES : X position';
	const SEM_MESSAGES_POS_Y	= '00d. MESSAGES : Y position';
	const SEM_MESSAGES_SCALE	= '00e. MESSAGES : scale';
	
	// RANKING
	const SEM_RANKING_POS_X	= '00f. RANKING : X position';
	const SEM_RANKING_POS_Y	= '00g. RANKING : Y position';
	const SEM_RANKING_SCALE	= '00h. RANKING : scale';
	
	// IGNORED_PLAYERS_LIST parameter
	const IGNORED_PLAYERS_LIST	= '00i. IGNORE_LIST';
	
	// PLAYER_CONNECTED parameters
	const ENABLE_PLAYER_CONNECTED_SOUND	= '01a. PLAYER_CONNECTED enabled ?';
	const SOUND_LINK_PLAYER_CONNECTED	= '01b. PLAYER_CONNECTED sound';
	
	// PLAYER_DISCONNECTED parameters
	const ENABLE_PLAYER_DISCONNECTED_SOUND	= '02a. PLAYER_DISCONNECTED enabled ?';
	const SOUND_LINK_PLAYER_DISCONNECTED	= '02b. PLAYER_DISCONNECTED sound';
	
	// BEGIN_MAP parameters
	const ENABLE_BEGIN_MAP_SOUND	= '03a. BEGIN_MAP enabled ?';
	const SOUND_LINK_BEGIN_MAP		= '03b. BEGIN_MAP sound';
	
	// BEGIN_PLAYING parameters
	const ENABLE_BEGIN_PLAYING_SOUND	= '04a. BEGIN_PLAYING enabled ?';
	const SOUND_LINK_BEGIN_PLAYING		= '04b. BEGIN_PLAYING sound';
	
	// BLUE/RED TEAM SCORES parameters
	const ENABLE_TEAM_SCORES_SOUND		= '05a. TEAM_SCORES enabled ?';
	const SOUND_LINK_BLUE_TEAM_SCORES	= '05b. BLUE_TEAM_SCORES sound';
	const SOUND_LINK_RED_TEAM_SCORES	= '05c. RED_TEAM_SCORES sound';
	
	// BACKSPACE parameters
	const ENABLE_BACKSPACE_SOUND	= '06a. BACKSPACE enabled ?';
	const SOUND_LINK_BACKSPACE		= '06b. BACKSPACE sound';
	
	// OFFZONE parameters
	const ENABLE_OFFZONE_SOUND	= '07a. OFF-ZONE enabled ?';
	const SOUND_LINK_OFFZONE	= '07b. OFF-ZONE sound';
	
	// HIT parameters
	const ENABLE_HIT_SOUND	= '08c. HIT enabled ?';
	const SOUND_LINK_HIT	= '08d. HIT sound';
	
	// TEAMKILLER parameters
	const ENABLE_TEAMKILLER_SOUND	= '09a. TEAMKILLER enabled ?';
	const SOUND_LINK_TEAMKILLER	= '09b. TEAMKILLER sound';
	
	// KILL AWARDS parameters
	const ENABLE_KILL_AWARDS	= '100. KILLSTREAK (consecutive kills without dying) enabled ?';
	
	const SOUND_LINK_KILL_01	= '100.01. Sound for kill 01';
	const SOUND_LINK_KILL_02	= '100.02. Sound for kill 02';
	const SOUND_LINK_KILL_03	= '100.03. Sound for kill 03';
	const SOUND_LINK_KILL_04	= '100.04. Sound for kill 04';
	const SOUND_LINK_KILL_05	= '100.05. Sound for kill 05';
	const SOUND_LINK_KILL_06	= '100.06. Sound for kill 06';
	const SOUND_LINK_KILL_07	= '100.07. Sound for kill 07';
	const SOUND_LINK_KILL_08	= '100.08. Sound for kill 08';
	const SOUND_LINK_KILL_09	= '100.09. Sound for kill 09';
	const SOUND_LINK_KILL_10	= '100.10. Sound for kill 10';
	const SOUND_LINK_KILL_11	= '100.11. Sound for kill 11';
	const SOUND_LINK_KILL_12	= '100.12. Sound for kill 12';
	const SOUND_LINK_KILL_13	= '100.13. Sound for kill 13';
	const SOUND_LINK_KILL_14	= '100.14. Sound for kill 14';
	const SOUND_LINK_KILL_15	= '100.15. Sound for kill 15';
	const SOUND_LINK_KILL_16	= '100.16. Sound for kill 16';
	const SOUND_LINK_KILL_17	= '100.17. Sound for kill 17';
	const SOUND_LINK_KILL_18	= '100.18. Sound for kill 18';
	const SOUND_LINK_KILL_19	= '100.19. Sound for kill 19';
	const SOUND_LINK_KILL_20	= '100.20. Sound for kill 20';
	const SOUND_LINK_KILL_21	= '100.21. Sound for kill 21';
	const SOUND_LINK_KILL_22	= '100.22. Sound for kill 22';
	const SOUND_LINK_KILL_23	= '100.23. Sound for kill 23';
	const SOUND_LINK_KILL_24	= '100.24. Sound for kill 24';
	const SOUND_LINK_KILL_25	= '100.25. Sound for kill 25';
	const SOUND_LINK_KILL_26	= '100.26. Sound for kill 26';
	const SOUND_LINK_KILL_27	= '100.27. Sound for kill 27';
	const SOUND_LINK_KILL_28	= '100.28. Sound for kill 28';
	const SOUND_LINK_KILL_29	= '100.29. Sound for kill 29';
	const SOUND_LINK_KILL_30	= '100.30. Sound for kill 30';
	
	
	
    /**
     * Private Properties
     */
    private $maniaControl = null;
	
	// SEM
	public $playersScore = array();
	public $disabledSoundsPlayers = array();
	public $playerTeamId;
	public $scoreMin;
	public $soundLinkMin;
	public $scoreMax;
	public $soundLinkMax;
	
	
	
	
	/*
	 * Check if a sound should be played. If so, plays that sound.
	 */
	public function soundToPlay($allowed, $link, $authorLogin) {
		$SOUND_ALLOWED	= $this->maniaControl->settingManager->getSettingValue($this, $allowed);
		$SOUND_LINK		= $this->maniaControl->settingManager->getSettingValue($this, $link);
		
		// Play sounds only if ALLOWED and LINK available and PLAYER NOT IGNORED
		if ($SOUND_ALLOWED == true && !empty($SOUND_LINK)) {
			$rc = $this->playSound($SOUND_LINK, $authorLogin);
			return $rc;
		}
		else { return false; }
	}
	
	
	
	/*
	 * Plays the sound "$link" if able
	 *
	 * Returns true if a sound has been played, false otherwise
	 *
	 */
	public function playSound($link, $authorLogin) {
		$VOLUME	= $this->maniaControl->settingManager->getSettingValue($this, self::VOLUME);
		
		$this->toLog("link=$link / authorLogin=$authorLogin / VOLUME=$VOLUME");
		
		// Don't play sounds if $link doesn't exist or author of sound is IGNORED
		if (empty($link) || $this->isIgnored($authorLogin)) { return false; }
		
		// Create an audio element to play a sound with the FML
		$maniaLink = new \FML\ManiaLink(self::ML_ID);
		$audio = new \FML\Controls\Audio();
		$maniaLink->add($audio);
		$audio->setVolume($VOLUME);		//						  from FML/Types/Playable.php
		$audio->setPlay(1);				// Play automatically	- from FML/Types/Playable.php
		$audio->setLooping(0);			// Play only once		- from FML/Types/Playable.php
		$audio->setVisible(0);			// Popup not visible	- from FML/Controls/Control.php
		$audio->setData($link);
		
		// Send ManiaLink
		$this->sendManiaLink($maniaLink);
		
		return true;
	}
	
	
	
	/**
	 * Display a message during a delay
	 *
	 */
	public function showMessage($message, $authorLogin) {
		// Remove the previous timer listener to keep the delay before removing the message
        $this->maniaControl->timerManager->unregisterTimerListening($this, 'removeMessage');
		
		// Don't display message if author of sound is IGNORED
		if ($this->isIgnored($authorLogin)) { return false; }
		
		$maniaLink = new ManiaLink(self::ML_MESSAGE_ID);

		// Main frame
		$frame = new Frame();
		$maniaLink->add($frame);
		$xPos	= $this->maniaControl->settingManager->getSettingValue($this, self::SEM_MESSAGES_POS_X);
		$yPos	= $this->maniaControl->settingManager->getSettingValue($this, self::SEM_MESSAGES_POS_Y);
		$scale	= $this->maniaControl->settingManager->getSettingValue($this, self::SEM_MESSAGES_SCALE);
		$frame->setPosition($xPos, $yPos, 0);

		// Background
		$backgroundQuad = new Quad();
		$frame->add($backgroundQuad);
		
		// Message
		$messageLabel = new Label_Button();
		$frame->add($messageLabel);
		$messageLabel->setScale($scale);
		$messageLabel->setText($message);
		$messageLabel->setStyle(Label_Text::STYLE_TextStaticSmall);
		
		// Send ManiaLink
		$this->sendManiaLink($maniaLink);
		
		// Remove the message after a delay
		$this->maniaControl->timerManager->registerOneTimeListening($this, 'removeMessage', 3000);
	}
	
	/*
	 * Remove the message
	 */
	public function removeMessage() {
		$this->maniaControl->manialinkManager->hideManialink(self::ML_MESSAGE_ID);
	}
	
	
	
	/**
	 * Popup a widget with the killstreak ranking of 3 best players
	 */
	public function showRanking($killStreak_1_player, $killStreak_1_score, $killStreak_2_player, $killStreak_2_score, $killStreak_3_player, $killStreak_3_score) {
		// Remove the widget
		$this->maniaControl->manialinkManager->hideManialink(self::ML_RANKING_ID);
		
		// Check if there's at least 1 ranking to display
		if ($killStreak_1_player == null) { return; }
		
		$maniaLink = new ManiaLink(self::ML_RANKING_ID);

		// Main frame
		$frame = new Frame();
		$maniaLink->add($frame);
		$xPos	= $this->maniaControl->settingManager->getSettingValue($this, self::SEM_RANKING_POS_X);
		$yPos	= $this->maniaControl->settingManager->getSettingValue($this, self::SEM_RANKING_POS_Y);
		$scale	= $this->maniaControl->settingManager->getSettingValue($this, self::SEM_RANKING_SCALE);
		$frame->setPosition($xPos, $yPos, 0);

		// Background
		$backgroundQuad = new Quad();
		$frame->add($backgroundQuad);
		
		// Ranking
		/*
		$messageLabel_0 = new Label_Button();
		$frame->add($messageLabel_0);
		$messageLabel_0->setPosition(1, 1, 1);
		$messageLabel_0->setScale($scale);
		$messageLabel_0->setAlign('left', 'center');
		$messageLabel_0->setText('$b$fff RANKING');
		$messageLabel_0->setStyle(Label_Text::STYLE_TextStaticSmall);
		*/
		
		// Ranking 1
		$killStreak_1_player_nickname = $this->maniaControl->getPlayerManager()->getPlayer($killStreak_1_player)->nickname;
		$messageLabel_1 = new Label_Button();
		$frame->add($messageLabel_1);
		$messageLabel_1->setPosition(1, -5, 2);
		$messageLabel_1->setScale($scale);
		$messageLabel_1->setAlign('left', 'center');
		$messageLabel_1->setText($killStreak_1_player_nickname . '$z$s$fff : ' . $killStreak_1_score . ' frags');
		$messageLabel_1->setStyle(Label_Text::STYLE_TextStaticSmall);
		
		// Ranking 2
		if ($killStreak_2_player !== null) {
			$killStreak_2_player_nickname = $this->maniaControl->getPlayerManager()->getPlayer($killStreak_2_player)->nickname;
			$messageLabel_2 = new Label_Button();
			$frame->add($messageLabel_2);
			$messageLabel_2->setPosition(1, -10, 3);
			$messageLabel_2->setScale($scale);
			$messageLabel_2->setAlign('left', 'center');
			$messageLabel_2->setText($killStreak_2_player_nickname . '$z$s$fff : ' . $killStreak_2_score . ' frags');
			$messageLabel_2->setStyle(Label_Text::STYLE_TextStaticSmall);
		}
		
		// Ranking 3
		if ($killStreak_3_player !== null) {
			$killStreak_3_player_nickname = $this->maniaControl->getPlayerManager()->getPlayer($killStreak_3_player)->nickname;
			$messageLabel_3 = new Label_Button();
			$frame->add($messageLabel_3);
			$messageLabel_3->setPosition(1, -15, 4);
			$messageLabel_3->setScale($scale);
			$messageLabel_3->setAlign('left', 'center');
			$messageLabel_3->setText($killStreak_3_player_nickname . '$z$s$fff : ' . $killStreak_3_score . ' frags');
			$messageLabel_3->setStyle(Label_Text::STYLE_TextStaticSmall);
		}
		
		// Send ManiaLink
		$this->sendManiaLink($maniaLink);
	}
	
	
	
	/**
	 * Send a ManiaLink to players allowed
	 */
	public function sendManiaLink($maniaLink) {
		// Plays sounds only for players who haven't disabled SEM and non-ignored players
		foreach ($this->maniaControl->getPlayerManager()->getPlayers() as $player) {
			$playerLogin = $player->login;
			if (!$this->hasDisabled($playerLogin) && !$this->isIgnored($playerLogin)) {
				$this->maniaControl->manialinkManager->sendManialink($maniaLink,$playerLogin);
			}
		}
	}
	
	
	
    /**
     * Fonction de log pour debugg
     *
     * @param $log
     */
    public function toLog($log) {
		$DEBUG = $this->maniaControl->settingManager->getSettingValue($this, self::DEBUG);
		if ($DEBUG == true) {
			$this->maniaControl->getChat()->sendInformation('[Sound Events Manager][' . $this->getVersion() . '] ' . $log, null, false);
		}
    }
	
	
	
	/*
	 * Initialize the array which stores players' scores
	 */
	public function initializeAllPlayers() {
		// Initialisation of playersScore[]
		foreach ($this->maniaControl->getPlayerManager()->getPlayers() as $player) {
			$playerLogin = $player->login;
			$this->playersScore[$playerLogin] = '0';
		}
	}
	
	
	
    /**
     * Prepares the Plugin
     *
     * @param ManiaControl $maniaControl
     */
    public static function prepare(ManiaControl $maniaControl) {
		//$this->toLog('into prepare()');
        //do nothing
    }
	
	/**
	 * Unload the plugin and its resources
	 */
    public function unload() {
		// DEBUG
		$this->toLog('Unloading Sound Events Manager...');
		
		// Sound icon
		$this->removeToMenu();
		
		// Callbacks
        $this->maniaControl->getCallbackManager()->unregisterCallbackListener($this);
    }
	
    /**
     * Load the plugin
     *
     * @param \ManiaControl\ManiaControl $maniaControl
     */
    public function load(ManiaControl $maniaControl) {
        $this->maniaControl = $maniaControl;
		
		$this->toLog('Loading Sound Events Manager...');
		
		// Detecting mode		
		// Cf http://mania-actu.com/browser/servers.php
		//    'SMStormElite@nadeolabs'
		//    'SMStormRoyal@nadeolabs'
		//    'SMStormSiege@nadeolabs'
		//    'SpeedBall@steeffeen'
		$mode = $this->maniaControl->server->titleId;
		$this->toLog('Mode is ' . $mode);
		
		// Global variables
		$rep_sounds = 'http://mm2monde.fr/Plugins/SoundEventsManager/UnrealTournamentSounds/';
		$this->scoreMin = 666;
		$this->soundLinkMin = '';
		$this->scoreMax = 666;;
		$this->soundLinkMax = '';
		
        //Register callbacks
		$this->maniaControl->getCallbackManager()->registerCallbackListener(Callbacks::BEGINMAP, $this, 'onBeginMap');
		$this->maniaControl->getCallbackManager()->registerCallbackListener(Callbacks::BEGINPLAYING, $this, 'onBeginPlaying');
		$this->maniaControl->getCallbackManager()->registerCallbackListener(Callbacks::ONCAPTURE, $this, 'onCapture');
		$this->maniaControl->getCallbackManager()->registerCallbackListener(Callbacks::ONPLAYERREQUESTRESPAWN, $this, 'onRequestRespawn');
		//$this->maniaControl->getCallbackManager()->registerCallbackListener(Callbacks::ONSHOOT, $this, 'onShoot');
		$this->maniaControl->getCallbackManager()->registerCallbackListener(Callbacks::ONHIT, $this, 'onHit');
		$this->maniaControl->getCallbackManager()->registerCallbackListener(Callbacks::ONARMOREMPTY, $this, 'onArmorEmpty');
		
		$this->maniaControl->getCallbackManager()->registerScriptCallbackListener('SpeedBall_OnPass', $this, 'onPass');
		
		$this->maniaControl->callbackManager->registerCallbackListener(PlayerManager::CB_PLAYERCONNECT, $this, 'handlePlayerConnect');
		$this->maniaControl->callbackManager->registerCallbackListener(PlayerManager::CB_PLAYERDISCONNECT, $this, 'handlePlayerDisconnect');
		
		
		// DEBUG messages
		$this->maniaControl->settingManager->initSetting($this, self::DEBUG, false);
		
		// VOLUME sound
		$this->maniaControl->settingManager->initSetting($this, self::VOLUME, 1);
		
		// MESSAGES
		$this->maniaControl->settingManager->initSetting($this, self::SEM_MESSAGES_POS_X, 0);
		$this->maniaControl->settingManager->initSetting($this, self::SEM_MESSAGES_POS_Y, 40);
		$this->maniaControl->settingManager->initSetting($this, self::SEM_MESSAGES_SCALE, 1.5);
		
		// RANKING
		$this->maniaControl->settingManager->initSetting($this, self::SEM_RANKING_POS_X, 110);
		$this->maniaControl->settingManager->initSetting($this, self::SEM_RANKING_POS_Y, 50);
		$this->maniaControl->settingManager->initSetting($this, self::SEM_RANKING_SCALE, 1);
		
		// IGNORED_PLAYERS_LIST Setting
		$this->maniaControl->settingManager->initSetting($this, self::IGNORED_PLAYERS_LIST, 'wellef;login1;login2;...');
		
		// PLAYER_CONNECTED Settings
		$this->maniaControl->settingManager->initSetting($this, self::ENABLE_PLAYER_CONNECTED_SOUND, false);
		$this->maniaControl->settingManager->initSetting($this, self::SOUND_LINK_PLAYER_CONNECTED, '');
		
		// PLAYER_DISCONNECTED Settings
		$this->maniaControl->settingManager->initSetting($this, self::ENABLE_PLAYER_DISCONNECTED_SOUND, false);
		$this->maniaControl->settingManager->initSetting($this, self::SOUND_LINK_PLAYER_DISCONNECTED, '');
		
		// BEGIN_MAP : allow and link
		$this->maniaControl->settingManager->initSetting($this, self::ENABLE_BEGIN_MAP_SOUND, false);
		$this->maniaControl->settingManager->initSetting($this, self::SOUND_LINK_BEGIN_MAP, '');
		
		// BEGIN_PLAYING : allow and link
		$this->maniaControl->settingManager->initSetting($this, self::ENABLE_BEGIN_PLAYING_SOUND, true);
		$this->maniaControl->settingManager->initSetting($this, self::SOUND_LINK_BEGIN_PLAYING, $rep_sounds . 'prepare.wav');
		
		// BLUE/RED TEAM SCORES : links
		$this->maniaControl->settingManager->initSetting($this, self::SOUND_LINK_BLUE_TEAM_SCORES, $rep_sounds . 'Blue_Team_Scores.wav');
		$this->maniaControl->settingManager->initSetting($this, self::SOUND_LINK_RED_TEAM_SCORES, $rep_sounds . 'Red_Team_Scores.wav');
				
		// BACKSPACE : allow and link
		$this->maniaControl->settingManager->initSetting($this, self::ENABLE_BACKSPACE_SOUND, true);
		$this->maniaControl->settingManager->initSetting($this, self::SOUND_LINK_BACKSPACE, $rep_sounds . 'ReturnSound.wav');
		
		// OFFZONE : allow and link
		$this->maniaControl->settingManager->initSetting($this, self::ENABLE_OFFZONE_SOUND, true);
		$this->maniaControl->settingManager->initSetting($this, self::SOUND_LINK_OFFZONE, $rep_sounds . 'humiliation.wav');
		
		// HIT : allow and link
		$this->maniaControl->settingManager->initSetting($this, self::ENABLE_HIT_SOUND, false);
		$this->maniaControl->settingManager->initSetting($this, self::SOUND_LINK_HIT, '');
		
		// TEAMKILLER : link
		$this->maniaControl->settingManager->initSetting($this, self::SOUND_LINK_TEAMKILLER, $rep_sounds . 'teamkiller.wav');
		
		// KILL AWARDS : allow
		$this->maniaControl->settingManager->initSetting($this, self::ENABLE_KILL_AWARDS, true);
		
		// Mode specific
		switch ($mode) {
			// SpeedBall default config
			case 'SpeedBall@steeffeen': {
				// BLUE/RED TEAM SCORES : links
				$this->maniaControl->settingManager->initSetting($this, self::ENABLE_TEAM_SCORES_SOUND, true);
				
				// TEAMKILLER : allow and link
				$this->maniaControl->settingManager->initSetting($this, self::ENABLE_TEAMKILLER_SOUND, true);
				
				// KILL AWARDS : links
				$this->maniaControl->settingManager->initSetting($this, self::SOUND_LINK_KILL_01, '');
				$this->maniaControl->settingManager->initSetting($this, self::SOUND_LINK_KILL_02, '');
				$this->maniaControl->settingManager->initSetting($this, self::SOUND_LINK_KILL_03, $rep_sounds . 'dominating.wav   ');
				$this->maniaControl->settingManager->initSetting($this, self::SOUND_LINK_KILL_04, $rep_sounds . 'rampage.wav      ');
				$this->maniaControl->settingManager->initSetting($this, self::SOUND_LINK_KILL_05, $rep_sounds . 'killingspree.wav ');
				$this->maniaControl->settingManager->initSetting($this, self::SOUND_LINK_KILL_06, $rep_sounds . 'ultrakill.wav    ');
				$this->maniaControl->settingManager->initSetting($this, self::SOUND_LINK_KILL_07, $rep_sounds . 'wickedsick.wav   ');
				$this->maniaControl->settingManager->initSetting($this, self::SOUND_LINK_KILL_08, $rep_sounds . 'monsterkill.wav  ');
				$this->maniaControl->settingManager->initSetting($this, self::SOUND_LINK_KILL_09, $rep_sounds . 'unstoppable.wav  ');
				$this->maniaControl->settingManager->initSetting($this, self::SOUND_LINK_KILL_10, $rep_sounds . 'godlike.wav      ');
				$this->maniaControl->settingManager->initSetting($this, self::SOUND_LINK_KILL_11, $rep_sounds . 'holyshit.wav     ');
				$this->maniaControl->settingManager->initSetting($this, self::SOUND_LINK_KILL_12, $rep_sounds . 'LudicrousKill.wav');
				$this->maniaControl->settingManager->initSetting($this, self::SOUND_LINK_KILL_13, '');
				$this->maniaControl->settingManager->initSetting($this, self::SOUND_LINK_KILL_14, '');
				$this->maniaControl->settingManager->initSetting($this, self::SOUND_LINK_KILL_15, '');
				$this->maniaControl->settingManager->initSetting($this, self::SOUND_LINK_KILL_16, '');
				$this->maniaControl->settingManager->initSetting($this, self::SOUND_LINK_KILL_17, '');
				$this->maniaControl->settingManager->initSetting($this, self::SOUND_LINK_KILL_18, '');
				$this->maniaControl->settingManager->initSetting($this, self::SOUND_LINK_KILL_19, '');
				$this->maniaControl->settingManager->initSetting($this, self::SOUND_LINK_KILL_20, '');
				$this->maniaControl->settingManager->initSetting($this, self::SOUND_LINK_KILL_21, '');
				$this->maniaControl->settingManager->initSetting($this, self::SOUND_LINK_KILL_22, '');
				$this->maniaControl->settingManager->initSetting($this, self::SOUND_LINK_KILL_23, '');
				$this->maniaControl->settingManager->initSetting($this, self::SOUND_LINK_KILL_24, '');
				$this->maniaControl->settingManager->initSetting($this, self::SOUND_LINK_KILL_25, '');
				$this->maniaControl->settingManager->initSetting($this, self::SOUND_LINK_KILL_26, '');
				$this->maniaControl->settingManager->initSetting($this, self::SOUND_LINK_KILL_27, '');
				$this->maniaControl->settingManager->initSetting($this, self::SOUND_LINK_KILL_28, '');
				$this->maniaControl->settingManager->initSetting($this, self::SOUND_LINK_KILL_29, '');
				$this->maniaControl->settingManager->initSetting($this, self::SOUND_LINK_KILL_30, '');
				break;
			}
			case 'SMStormSiege@nadeolabs': {
				// BLUE/RED TEAM SCORES : allow
				$this->maniaControl->settingManager->initSetting($this, self::ENABLE_TEAM_SCORES_SOUND, false);
				
				// TEAMKILLER : allow
				$this->maniaControl->settingManager->initSetting($this, self::ENABLE_TEAMKILLER_SOUND, false);
				
				// KILL AWARDS : links
				$this->maniaControl->settingManager->initSetting($this, self::SOUND_LINK_KILL_01, '');
				$this->maniaControl->settingManager->initSetting($this, self::SOUND_LINK_KILL_02, $rep_sounds . 'dominating.wav   ');
				$this->maniaControl->settingManager->initSetting($this, self::SOUND_LINK_KILL_03, '');
				$this->maniaControl->settingManager->initSetting($this, self::SOUND_LINK_KILL_04, $rep_sounds . 'rampage.wav      ');
				$this->maniaControl->settingManager->initSetting($this, self::SOUND_LINK_KILL_05, '');
				$this->maniaControl->settingManager->initSetting($this, self::SOUND_LINK_KILL_06, $rep_sounds . 'killingspree.wav ');
				$this->maniaControl->settingManager->initSetting($this, self::SOUND_LINK_KILL_07, '');
				$this->maniaControl->settingManager->initSetting($this, self::SOUND_LINK_KILL_08, $rep_sounds . 'ultrakill.wav    ');
				$this->maniaControl->settingManager->initSetting($this, self::SOUND_LINK_KILL_09, '');
				$this->maniaControl->settingManager->initSetting($this, self::SOUND_LINK_KILL_10, $rep_sounds . 'wickedsick.wav   ');
				$this->maniaControl->settingManager->initSetting($this, self::SOUND_LINK_KILL_11, '');
				$this->maniaControl->settingManager->initSetting($this, self::SOUND_LINK_KILL_12, $rep_sounds . 'monsterkill.wav  ');
				$this->maniaControl->settingManager->initSetting($this, self::SOUND_LINK_KILL_13, '');
				$this->maniaControl->settingManager->initSetting($this, self::SOUND_LINK_KILL_14, $rep_sounds . 'unstoppable.wav  ');
				$this->maniaControl->settingManager->initSetting($this, self::SOUND_LINK_KILL_15, '');
				$this->maniaControl->settingManager->initSetting($this, self::SOUND_LINK_KILL_16, $rep_sounds . 'godlike.wav      ');
				$this->maniaControl->settingManager->initSetting($this, self::SOUND_LINK_KILL_17, '');
				$this->maniaControl->settingManager->initSetting($this, self::SOUND_LINK_KILL_18, $rep_sounds . 'holyshit.wav     ');
				$this->maniaControl->settingManager->initSetting($this, self::SOUND_LINK_KILL_19, '');
				$this->maniaControl->settingManager->initSetting($this, self::SOUND_LINK_KILL_20, $rep_sounds . 'LudicrousKill.wav');
				$this->maniaControl->settingManager->initSetting($this, self::SOUND_LINK_KILL_21, '');
				$this->maniaControl->settingManager->initSetting($this, self::SOUND_LINK_KILL_22, '');
				$this->maniaControl->settingManager->initSetting($this, self::SOUND_LINK_KILL_23, '');
				$this->maniaControl->settingManager->initSetting($this, self::SOUND_LINK_KILL_24, '');
				$this->maniaControl->settingManager->initSetting($this, self::SOUND_LINK_KILL_25, '');
				$this->maniaControl->settingManager->initSetting($this, self::SOUND_LINK_KILL_26, '');
				$this->maniaControl->settingManager->initSetting($this, self::SOUND_LINK_KILL_27, '');
				$this->maniaControl->settingManager->initSetting($this, self::SOUND_LINK_KILL_28, '');
				$this->maniaControl->settingManager->initSetting($this, self::SOUND_LINK_KILL_29, '');
				$this->maniaControl->settingManager->initSetting($this, self::SOUND_LINK_KILL_30, '');
				break;
			}
			default: {
				// BLUE/RED TEAM SCORES : allow
				$this->maniaControl->settingManager->initSetting($this, self::ENABLE_TEAM_SCORES_SOUND, true);
				
				// TEAMKILLER : allow
				$this->maniaControl->settingManager->initSetting($this, self::ENABLE_TEAMKILLER_SOUND, false);
				
				// KILL AWARDS : links
				$this->maniaControl->settingManager->initSetting($this, self::SOUND_LINK_KILL_01, '');
				$this->maniaControl->settingManager->initSetting($this, self::SOUND_LINK_KILL_02, '');
				$this->maniaControl->settingManager->initSetting($this, self::SOUND_LINK_KILL_03, $rep_sounds . 'dominating.wav   ');
				$this->maniaControl->settingManager->initSetting($this, self::SOUND_LINK_KILL_04, $rep_sounds . 'rampage.wav      ');
				$this->maniaControl->settingManager->initSetting($this, self::SOUND_LINK_KILL_05, $rep_sounds . 'killingspree.wav ');
				$this->maniaControl->settingManager->initSetting($this, self::SOUND_LINK_KILL_06, $rep_sounds . 'ultrakill.wav    ');
				$this->maniaControl->settingManager->initSetting($this, self::SOUND_LINK_KILL_07, $rep_sounds . 'wickedsick.wav   ');
				$this->maniaControl->settingManager->initSetting($this, self::SOUND_LINK_KILL_08, $rep_sounds . 'monsterkill.wav  ');
				$this->maniaControl->settingManager->initSetting($this, self::SOUND_LINK_KILL_09, $rep_sounds . 'unstoppable.wav  ');
				$this->maniaControl->settingManager->initSetting($this, self::SOUND_LINK_KILL_10, $rep_sounds . 'godlike.wav      ');
				$this->maniaControl->settingManager->initSetting($this, self::SOUND_LINK_KILL_11, $rep_sounds . 'holyshit.wav     ');
				$this->maniaControl->settingManager->initSetting($this, self::SOUND_LINK_KILL_12, $rep_sounds . 'LudicrousKill.wav');
				$this->maniaControl->settingManager->initSetting($this, self::SOUND_LINK_KILL_13, '');
				$this->maniaControl->settingManager->initSetting($this, self::SOUND_LINK_KILL_14, '');
				$this->maniaControl->settingManager->initSetting($this, self::SOUND_LINK_KILL_15, '');
				$this->maniaControl->settingManager->initSetting($this, self::SOUND_LINK_KILL_16, '');
				$this->maniaControl->settingManager->initSetting($this, self::SOUND_LINK_KILL_17, '');
				$this->maniaControl->settingManager->initSetting($this, self::SOUND_LINK_KILL_18, '');
				$this->maniaControl->settingManager->initSetting($this, self::SOUND_LINK_KILL_19, '');
				$this->maniaControl->settingManager->initSetting($this, self::SOUND_LINK_KILL_20, '');
				$this->maniaControl->settingManager->initSetting($this, self::SOUND_LINK_KILL_21, '');
				$this->maniaControl->settingManager->initSetting($this, self::SOUND_LINK_KILL_22, '');
				$this->maniaControl->settingManager->initSetting($this, self::SOUND_LINK_KILL_23, '');
				$this->maniaControl->settingManager->initSetting($this, self::SOUND_LINK_KILL_24, '');
				$this->maniaControl->settingManager->initSetting($this, self::SOUND_LINK_KILL_25, '');
				$this->maniaControl->settingManager->initSetting($this, self::SOUND_LINK_KILL_26, '');
				$this->maniaControl->settingManager->initSetting($this, self::SOUND_LINK_KILL_27, '');
				$this->maniaControl->settingManager->initSetting($this, self::SOUND_LINK_KILL_28, '');
				$this->maniaControl->settingManager->initSetting($this, self::SOUND_LINK_KILL_29, '');
				$this->maniaControl->settingManager->initSetting($this, self::SOUND_LINK_KILL_30, '');
				break;
			}
		}
		
		// Initialisation
		$this->initializeAllPlayers();
		
		// Menu icons to enable/disable sounds
		$this->maniaControl->manialinkManager->registerManialinkPageAnswerListener(self::ENABLE_DISABLE_SEM, $this, 'enable_disable_SEM');
		$this->maniaControl->manialinkManager->getIconManager()->addIcon(self::SEM_ICON_SOUND_ON			, self::SEM_ICON_SOUND_ON_PATH       );
		$this->maniaControl->manialinkManager->getIconManager()->addIcon(self::SEM_ICON_SOUND_ON_MOVER		, self::SEM_ICON_SOUND_ON_MOVER_PATH );
		$this->maniaControl->manialinkManager->getIconManager()->addIcon(self::SEM_ICON_SOUND_OFF			, self::SEM_ICON_SOUND_OFF_PATH      );
		$this->maniaControl->manialinkManager->getIconManager()->addIcon(self::SEM_ICON_SOUND_OFF_MOVER		, self::SEM_ICON_SOUND_OFF_MOVER_PATH);
		
		// Add icons to menu
		$this->addToMenu_Disable();
		$this->disabledSoundsPlayers = array();
		
    }
	
	
	
	/**
	 * Function handling the pressing of the icon.
	 *
	 * @param array  $chatCallback
	 * @param Player $player
	 */
	public function enable_disable_SEM(array $chatCallback, Player $player) {
		$playerLogin = $player->login;
		$playerNickname = $player->nickname;
		
		if (in_array($playerLogin, $this->disabledSoundsPlayers)) {
			// Log
			$this->toLog('Sound Events Manager enabled for ' . $playerNickname);
			$this->maniaControl->getChat()->sendInformation('[Sound Events Manager] Sounds enabled.', $playerLogin, false);
			// Remove the current button
			$this->removeToMenu();
			// Add the "disable" button
			$this->addToMenu_Disable();
			// Remove the player from the "disabledSoundsPlayers" list
			unset($this->disabledSoundsPlayers[$playerLogin]);
		}
		else {
			// Log
			$this->toLog('Sound Events Manager disabled for ' . $playerNickname);
			$this->maniaControl->getChat()->sendInformation('[Sound Events Manager] Sounds disabled.', $playerLogin, false);
			// Remove the current button
			$this->removeToMenu();
			// Add the "disable" button
			$this->addToMenu_Enable();
			// Add the player from the "disabledSoundsPlayers" list
			$this->disabledSoundsPlayers[$playerLogin] = $playerLogin;
		}
	
	}
	
	/**
	 * Add the "enable" button into the menu
	 */
	private function addToMenu_Enable() {
		$itemQuad = new Quad();
		$itemQuad->setImage($this->maniaControl->manialinkManager->getIconManager()->getIcon(self::SEM_ICON_SOUND_ON));
		$itemQuad->setImageFocus($this->maniaControl->manialinkManager->getIconManager()->getIcon(self::SEM_ICON_SOUND_ON_MOVER));
		$itemQuad->setAction(self::ENABLE_DISABLE_SEM);
		$this->maniaControl->actionsMenu->addMenuItem($itemQuad, true, 38, 'Enable sounds');
	}
	
	/**
	 * Add the "disable" button into the menu
	 */
	private function addToMenu_Disable() {
		$itemQuad = new Quad();
		$itemQuad->setImage($this->maniaControl->manialinkManager->getIconManager()->getIcon(self::SEM_ICON_SOUND_OFF));
		$itemQuad->setImageFocus($this->maniaControl->manialinkManager->getIconManager()->getIcon(self::SEM_ICON_SOUND_OFF_MOVER));
		$itemQuad->setAction(self::ENABLE_DISABLE_SEM);
		$this->maniaControl->actionsMenu->addMenuItem($itemQuad, true, 38, 'Disable sounds');
	}
	
	/*
	 * Remove the button from the menu
	 */
	public function removeToMenu() {
		$this->maniaControl->actionsMenu->removeMenuItem(38, true);
	}
	
	
	
	/*
	 * Return true is $login is ignored, false otherwise.
	 */
	public function isIgnored($login) {
		// No need to test is $login is not set
		if ($login == null) { return false; }
		
		// Get ignore list
		$IGNORED_PLAYERS_LIST	= $this->maniaControl->settingManager->getSettingValue($this, self::IGNORED_PLAYERS_LIST);
		
		// Get the list of ignored players in an array
		$listeOfIgnoredPlayers = array();
		if (!empty($IGNORED_PLAYERS_LIST)) { $listeOfIgnoredPlayers = explode(";", $IGNORED_PLAYERS_LIST); }
		
		// Return if $login is ignored
		if (!empty($listeOfIgnoredPlayers) && in_array($login, $listeOfIgnoredPlayers)) { return true; }
		else { return false; }
	}
	
	/*
	 * Return true is $login has disabled the plugin, false otherwise.
	 */
	public function hasDisabled($login) {
		if (in_array($login, $this->disabledSoundsPlayers)) { return true; }
		else { return false; }
	}
	
	
	
	/**
	 * Function handling on the connection of a player.
	 *
	 * @param Player $player
	 */
	public function handlePlayerConnect(Player $player) {
		$login = $player->login;
		// DEBUG
		$this->toLog('Hello ' . $player->nickname . '$z$s$fff (' . $login . ') !');
		// Initialization of the player connected
		$this->playersScore[$login] = '0';
		// Play sound ?
		$this->soundToPlay(self::ENABLE_PLAYER_CONNECTED_SOUND, self::SOUND_LINK_PLAYER_CONNECTED, $login);
	}
	
	/**
	 * Function handling on the disconnection of a player.
	 *
	 * @param array $callback
	 */
	public function handlePlayerDisconnect(Player $player) {
		$login = $player->login;
		// DEBUG
		$this->toLog('Bye ' . $player->nickname . '$z$s$fff (' . $login . ') !');
		// Play sound ?
		$this->soundToPlay(self::ENABLE_PLAYER_DISCONNECTED_SOUND, self::SOUND_LINK_PLAYER_DISCONNECTED, $login);
		// Removing informations from this player
		unset($this->playersScore[$login]);
	}
	
	
	
	/**
	 * Function called when starting a new map
	 */
	public function onBeginMap() {
		$this->toLog('Welcome to this new map !');
		$this->initializeAllPlayers();
		$this->ranking();
		$this->maniaControl->timerManager->registerOneTimeListening($this, 'playBeginMapSound', 2500);
	}
	
	/*
	 * Play BEGIN_MAP sound
	 */
	public function playBeginMapSound() {
		// Sound to play ?
		$this->soundToPlay(self::ENABLE_BEGIN_MAP_SOUND, self::SOUND_LINK_BEGIN_MAP, null);
	}
	
	
	
	/**
	 * Function called when starting a new round
	 */
	public function onBeginPlaying() {
		$this->toLog('Prepare to fight !');
		$this->maniaControl->timerManager->registerOneTimeListening($this, 'playBeginPlayingSound', 3000);
	}
	
	/*
	 * Play BEGING_PLAYING sound
	 */
	public function playBeginPlayingSound() {
		$this->soundToPlay(self::ENABLE_BEGIN_PLAYING_SOUND, self::SOUND_LINK_BEGIN_PLAYING, null);
	}
	
	
	
	/**
	 * Function called when some captures a pole
	 */
	// https://github.com/ManiaControl/ManiaControl/blob/master/core/Callbacks/Structures/CaptureStructure.php
	public function onCapture(CaptureStructure $structure) {
		foreach ($structure->getPlayerArray() as $player) {
			// DEBUG
			$this->toLog($player->nickname . '$z$s$fff : Nice capture !');
			
			// BLUE/RED TEAM SCORES
			$this->playerTeamId = $player->teamId;
			$this->toLog('Team ' . $this->playerTeamId . ' scores !');
			if (!$this->isIgnored($player->login)) {
				$this->maniaControl->timerManager->registerOneTimeListening($this, 'playTeamScoresSound', 1500); // OK
			}
		}
	}
	
	/*
	 * Play BLUE/RED TEAM SCORES sound
	 */
	public function playTeamScoresSound() {
		if ($this->playerTeamId == 0) {
			$this->soundToPlay(self::ENABLE_TEAM_SCORES_SOUND, self::SOUND_LINK_BLUE_TEAM_SCORES, null);
		}
		else {
			$this->soundToPlay(self::ENABLE_TEAM_SCORES_SOUND, self::SOUND_LINK_RED_TEAM_SCORES, null);
		}
	}
	
	
	
	/**
	 * Function called when someone BACKSPACE (+ suppr ?)
	 */
	public function onRequestRespawn(Player $player) {
		$playerNickname	= $player->nickname;
		$playerLogin	= $player->login;
		$message = $playerNickname . '$z$s$fff : Backspace !';
		// DEBUG
		$this->toLog($message);
		// Reset player's score
		$this->playersScore[$playerLogin] = '0';
		// Sound to play ?
		$rc = $this->soundToPlay(self::ENABLE_BACKSPACE_SOUND, self::SOUND_LINK_BACKSPACE, $playerLogin);
		// Message to display ?
		if ($rc) { $this->showMessage($playerNickname . '$z$s$fff : Backspace !', $playerLogin); }
		// Update RANKING
		$this->ranking();
		
		// Score of teams - TO DO
		// Récupérer les paramètres du mode (le temps de chaque phase) et le score de chaque équipe
		// 		S_TimeLimitNormal
		// 		S_TimeLimitReduction
		// 		S_TimeLimitInstant
		//	
		//$this->maniaControl->settingManager->getSetting
	}
	
	
	
	/*
	 * Function called when a player hit another player
	 */
	/*
	public function onShoot(Player $player) {
		$message = $player->nickname . '$z$s$fff : shoot';
		// DEBUG
		$this->toLog($message);
		// Message to display ?
		$this->showMessage($message, $player->login);
	}
	*/
	
	
	
	/*
	 * Function called when a player hit another player
	 */
	public function onHit(PlayerHitStructure $structure) {
		$shooter = $structure->getShooter();
		$victim = $structure->getVictim();
		$damage = $structure->getDamage();
		$shooterPoints = $structure->getShooterPoints();
		$weapon = $structure->getWeapon();
		$hitDistance = $structure->getHitDistance();
		
		// DEBUG
		$this->toLog($shooter->login . ' hit ' . $victim->login . ' on a distance of ' . $hitDistance);
		
		// Play sound ?
		$this->soundToPlay(self::ENABLE_HIT_SOUND, self::SOUND_LINK_HIT, $shooter->login);
	}
	
	
	
	/**
	 * Function called when a player has no more armor (killed by weapon, killed by offzone)
	 *
	 */
	// https://github.com/ManiaControl/ManiaControl/blob/master/core/Callbacks/Structures/ArmorEmptyStructure.php
	public function onArmorEmpty(ArmorEmptyStructure $structure) {
		// Get informations
		$shooter = $structure->getShooter();
		if ($shooter !== null) {
			$shooterLogin = $shooter->login;
			$shooterNickname = $shooter->nickname;
			$shooterTeamId = $shooter->teamId;
			$shooterScore = $this->playersScore[$shooterLogin];
		}
		$victim = $structure->getVictim();
		if ($victim !== null) {
			$victimLogin = $victim->login;
			$victimNickname = $victim->nickname;
			$victimTeamId = $victim->teamId;
			$victimScore = $this->playersScore[$victimLogin];
		}
		$damage = $structure->getDamage();
		$shooterPoints = $structure->getShooterPoints();
		$weapon = $structure->getWeapon();
		
		// DEBUG
		// $msg = 'onArmorEmpty() : shooter=' . $structure->getShooter() . ' / victim=' . $structure->getVictim() . ' / damage=' . $structure->getDamage() . ' / shooterPoints=' . $structure->getShooterPoints() . ' / weapon=' . $structure->getWeapon();
		if ($shooter !== null && $victim !== null) { $this->toLog($shooterNickname . ' killed ' . $victimNickname . ' !'); }
		
		// KILL BY PLAYER
		if ($shooter !== null) {
			// TEAMKILLER
			if ($shooterTeamId == $victimTeamId) {
				$rc = $this->soundToPlay(self::ENABLE_TEAMKILLER_SOUND, self::SOUND_LINK_TEAMKILLER, $shooterLogin);
				if ($rc) { $this->showMessage($shooterNickname . '$z$s$fff : Team killer !', $shooterLogin); }
			}
			// KILLSTREAK
			else {
				$this->playersScore[$shooterLogin] = $shooterScore + 1;
				
				// Sound to play ?
				$this->killAwards($shooter);
				
				// Delai on killAwards : KO, can't pass parameter
				// File&Line: /srv/shootmania/newspeedball/maniacontrol/core/Callbacks/TimerManager.php: 54
				// [18-Jan-2015 14:15:35 UTC] PHP Notice:  Given Listener (Lumpik\SoundEventsManager) can't handle Timer Callback (No Method 'Lumpik\killAwards(murphyslaw)')! in /srv/shootmania/newspeedball/maniacontrol/core/Callbacks/TimerManager.php on line 54
				// [18-Jan-2015 14:15:38 UTC] [ManiaControl NOTICE]: Given Listener (Lumpik\SoundEventsManager) can't handle Timer Callback (No Method '')!
				// File&Line: /srv/shootmania/newspeedball/maniacontrol/core/Callbacks/TimerManager.php: 54
				// [18-Jan-2015 14:15:38 UTC] PHP Notice:  Given Listener (Lumpik\SoundEventsManager) can't handle Timer Callback (No Method '')! in /srv/shootmania/newspeedball/maniacontrol/core/Callbacks/TimerManager.php on line 54
				//$this->maniaControl->timerManager->registerOneTimeListening($this, "Lumpik\killAwards($shooterLogin)", 500);
			}
		}
		
		// DEATH
		if ($victim !== null) {
			$this->playersScore[$victimLogin] = '0';
			
			// DEATH BY OFF-ZONE
			if ($shooter === null) {
				$this->toLog($victimNickname . ' $z$s$fff : Off-zoned !');
				$rc = $this->soundToPlay(self::ENABLE_OFFZONE_SOUND, self::SOUND_LINK_OFFZONE, $victimLogin);
				if ($rc) { $this->showMessage($victimNickname . '$z$s$fff : Off-zoned !', $victimLogin); }
			}
		}
		
		// Update RANKING
		$this->ranking();
	}
	
	
	
	/**
	 * Get the ranking of the 3 best killstreakers and display a widget of killstreak ranking
	 */
	public function ranking() {
		$killStreak_1_player	= null;
		$killStreak_1_score		= 0;
		$killStreak_2_player	= null;
		$killStreak_2_score		= 0;
		$killStreak_3_player	= null;
		$killStreak_3_score		= 0;
		
		// Sort array by descending scores
		arsort($this->playersScore);
		$i = 0;
		// Get the 3 best killstreak players with a score >= 1st score to play sounds
		foreach ($this->playersScore as $player => $score) {
			// Don't treat IGNORED players
			if ($this->isIgnored($player)){ break; }
			
			// Don't treat scores lower than the minimum score needed to play the 1st sound
			$this->toLog('score=' . $score . ' for ' . $player . ' (min=' . $this->scoreMin . ' - max=' . $this->scoreMax . ')');
			if ($score < $this->scoreMin) { break; }
			
			$i = $i + 1;
			
			// 1st best killstreak player
			if ($i == 1) {
				$killStreak_1_player	= $player;
				$killStreak_1_score		= $score;
			}
			// 2nd best killstreak player
			else if ($i == 2) {
				$killStreak_2_player	= $player;
				$killStreak_2_score		= $score;
			}
			// 3rd best killstreak player
			else if ($i == 3) {
				$killStreak_3_player	= $player;
				$killStreak_3_score		= $score;
			}
			
			// Get only 3 best players
			if ($i == 3) { break; }
		}
		
		// Display the widget of killstreak ranking
		$this->showRanking($killStreak_1_player, $killStreak_1_score, $killStreak_2_player, $killStreak_2_score, $killStreak_3_player, $killStreak_3_score);
	}
	
	
	
	/**
	 * Play sounds if the $player has a good score
	 *
	 * @param Player player
	 */
	public function killAwards(Player $player) {
		$playerLogin = $player->login;
		$playerNickname = $player->nickname;
		
		// Kill awards activated ?
		$ENABLE_KILL_AWARDS = $this->maniaControl->settingManager->getSettingValue($this, self::ENABLE_KILL_AWARDS);
		if ($ENABLE_KILL_AWARDS == false) { return; }
		
		// Get parameters
		// NB : can't use dynamic variables for "self::SOUND_LINK_KILL_XX"
		$SOUND_LINK_KILL_01 = $this->maniaControl->settingManager->getSettingValue($this, self::SOUND_LINK_KILL_01);
		$SOUND_LINK_KILL_02 = $this->maniaControl->settingManager->getSettingValue($this, self::SOUND_LINK_KILL_02);
		$SOUND_LINK_KILL_03 = $this->maniaControl->settingManager->getSettingValue($this, self::SOUND_LINK_KILL_03);
		$SOUND_LINK_KILL_04 = $this->maniaControl->settingManager->getSettingValue($this, self::SOUND_LINK_KILL_04);
		$SOUND_LINK_KILL_05 = $this->maniaControl->settingManager->getSettingValue($this, self::SOUND_LINK_KILL_05);
		$SOUND_LINK_KILL_06 = $this->maniaControl->settingManager->getSettingValue($this, self::SOUND_LINK_KILL_06);
		$SOUND_LINK_KILL_07 = $this->maniaControl->settingManager->getSettingValue($this, self::SOUND_LINK_KILL_07);
		$SOUND_LINK_KILL_08 = $this->maniaControl->settingManager->getSettingValue($this, self::SOUND_LINK_KILL_08);
		$SOUND_LINK_KILL_09 = $this->maniaControl->settingManager->getSettingValue($this, self::SOUND_LINK_KILL_09);
		$SOUND_LINK_KILL_10 = $this->maniaControl->settingManager->getSettingValue($this, self::SOUND_LINK_KILL_10);
		$SOUND_LINK_KILL_11 = $this->maniaControl->settingManager->getSettingValue($this, self::SOUND_LINK_KILL_11);
		$SOUND_LINK_KILL_12 = $this->maniaControl->settingManager->getSettingValue($this, self::SOUND_LINK_KILL_12);
		$SOUND_LINK_KILL_13 = $this->maniaControl->settingManager->getSettingValue($this, self::SOUND_LINK_KILL_13);
		$SOUND_LINK_KILL_14 = $this->maniaControl->settingManager->getSettingValue($this, self::SOUND_LINK_KILL_14);
		$SOUND_LINK_KILL_15 = $this->maniaControl->settingManager->getSettingValue($this, self::SOUND_LINK_KILL_15);
		$SOUND_LINK_KILL_16 = $this->maniaControl->settingManager->getSettingValue($this, self::SOUND_LINK_KILL_16);
		$SOUND_LINK_KILL_17 = $this->maniaControl->settingManager->getSettingValue($this, self::SOUND_LINK_KILL_17);
		$SOUND_LINK_KILL_18 = $this->maniaControl->settingManager->getSettingValue($this, self::SOUND_LINK_KILL_18);
		$SOUND_LINK_KILL_19 = $this->maniaControl->settingManager->getSettingValue($this, self::SOUND_LINK_KILL_19);
		$SOUND_LINK_KILL_20 = $this->maniaControl->settingManager->getSettingValue($this, self::SOUND_LINK_KILL_20);
		$SOUND_LINK_KILL_21 = $this->maniaControl->settingManager->getSettingValue($this, self::SOUND_LINK_KILL_21);
		$SOUND_LINK_KILL_22 = $this->maniaControl->settingManager->getSettingValue($this, self::SOUND_LINK_KILL_22);
		$SOUND_LINK_KILL_23 = $this->maniaControl->settingManager->getSettingValue($this, self::SOUND_LINK_KILL_23);
		$SOUND_LINK_KILL_24 = $this->maniaControl->settingManager->getSettingValue($this, self::SOUND_LINK_KILL_24);
		$SOUND_LINK_KILL_25 = $this->maniaControl->settingManager->getSettingValue($this, self::SOUND_LINK_KILL_25);
		$SOUND_LINK_KILL_26 = $this->maniaControl->settingManager->getSettingValue($this, self::SOUND_LINK_KILL_26);
		$SOUND_LINK_KILL_27 = $this->maniaControl->settingManager->getSettingValue($this, self::SOUND_LINK_KILL_27);
		$SOUND_LINK_KILL_28 = $this->maniaControl->settingManager->getSettingValue($this, self::SOUND_LINK_KILL_28);
		$SOUND_LINK_KILL_29 = $this->maniaControl->settingManager->getSettingValue($this, self::SOUND_LINK_KILL_29);
		$SOUND_LINK_KILL_30 = $this->maniaControl->settingManager->getSettingValue($this, self::SOUND_LINK_KILL_30);
		
		// Last SOUND_LINK and KILLSTREAK associated ?
		// For all 30 variables, starting from the last...
		for ($i = 30; $i >= 0; $i--) {
			// Get $i in 2 digits
			$i_2digits = sprintf("%02u",$i);
			// if the "SOUND_LINK_KILL" variable is not empty...
			if (!empty(${"SOUND_LINK_KILL_$i_2digits"})) {
				// Store SOUND_LINK and KILLSTREAK associated
				$this->scoreMax = $i;
				$this->soundLinkMax = ${"SOUND_LINK_KILL_$i_2digits"} ;
				// Stop looping
				break;
			}
		}
		
		// First SOUND_LINK and KILLSTREAK associated ?
		// For all 30 variables, starting from the first...
		for ($i = 1; $i <= 30 ; $i++) {
			// Get $i in 2 digits
			$i_2digits = sprintf("%02u",$i);
			// if the "SOUND_LINK_KILL" variable is not empty...
			if (!empty(${"SOUND_LINK_KILL_$i_2digits"})) {
				// Store SOUND_LINK and KILLSTREAK associated
				$this->scoreMin = $i;
				$this->soundLinkMin = ${"SOUND_LINK_KILL_$i_2digits"} ;
				// Stop looping
				break;
			}
		}
		
		// Play sound depending of player's score
		if (!is_null($playerLogin)) {
			$playerScore = $this->playersScore[$playerLogin];
			// Infinite "last-sound" if scoreMax hited
			if ($playerScore >= $this->scoreMax) { 
				$this->toLog($playerNickname . '$z$s$fff : Kill ' . $playerScore . ' !!');
				$rc = $this->playSound($this->soundLinkMax, $playerLogin);
				if ($rc) { $this->showMessage($playerNickname . '$z$s$fff : Kill ' . $playerScore . ' !!', $playerLogin); }
			}
			// Or just sounds depending on killstreak of the player
			else {
				$this->toLog($playerNickname . '$z$s$fff : Kill ' . $playerScore . ' !');
				$playerScore_2digits = sprintf("%02u",$playerScore);
				$SOUND_LINK = ${"SOUND_LINK_KILL_$playerScore_2digits"};
				$rc = $this->playSound($SOUND_LINK, $playerLogin);
				if ($rc) { $this->showMessage($playerNickname . '$z$s$fff : Kill ' . $playerScore . ' !', $playerLogin); }
			}
			
		}
	}
	
	
	
	
	
	
	
	
	
	// Cf SBXmlRpc::OnPass(_Event) from https://github.com/steeffeen/ManiaPlanet-Scripts/blob/master/Modes/ShootMania/SpeedBall.Script.txt
	// which calls onPass() from https://github.com/steeffeen/ManiaPlanet-Scripts/blob/master/Libs/steeffeen/ShootMania/SpeedBallXmlRpc.Script.txt
	public function onPass(Array $array) {
		$shooterLogin = $array[1][0];
		$victimLogin = $array[1][1];
		$shooterNickname = $this->maniaControl->getPlayerManager()->getPlayer($shooterLogin)->nickname;
		$victimNickname = $this->maniaControl->getPlayerManager()->getPlayer($victimLogin)->nickname;
		$this->toLog($shooterNickname . ' $z$s$fff : Nice pass to ' . $victimNickname . ' !');
	}
	
	
	
	
	
	
	/**
	 * Function called when a weapon (all ?) was near an enemy
	 */
	// https://github.com/ManiaControl/ManiaControl/blob/master/core/Callbacks/Structures/NearMissStructure.php
	// TO MODIF ?
	public function onNearMiss(NearMissStructure $structure) {
		//$msg = 'onNearMiss() : shooter=' . $structure->getShooter() . ' / victim=' . $structure->getVictim() . ' / distance=' . $structure->getDistance() . ' / weapon=' . $structure->getWeapon();
		//$this->toLog($msg);
	}
	
	
	
	
	
	
     /**
     * Get plugin id
     *
     * @return int
     */
    public static function getId() {
        return self::ID;
    }

    /**
     * Get Plugin Name
     *
     * @return string
     */
    public static function getName() {
        return self::NAME;
    }

    /**
     * Get Plugin Version
     *
     * @return float,,
     */
    public static function getVersion() {
        return self::VERSION;
    }

    /**
     * Get Plugin Author
     *
     * @return string
     */
    public static function getAuthor() {
        return self::AUTHOR;
    }

    /**
     * Get Plugin Description
     *
     * @return string
     */
    public static function getDescription() {
        return self::DESCRIPTION;
    }
}

